/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file Circle.cpp
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the Circle class implementation.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#include "Circle.h"

#include <math.h>

using namespace DelaunayTest;

Circle::Circle(const Point& center, float radius)	:	m_center(center), m_radius(radius)
{
}

Circle::Circle(const Point& p1, const Point& p2, const Point& p3)
{
	m_radius = CircleFrom3Points(p1, p2, p3, m_center);
}

Circle::~Circle(void)
{
}

/// @see http://home.att.net/~srschmitt/circle3pts.html
float Circle::CircleFrom3Points(const Point& p1, const Point& p2, const Point& p3, Point& center)
{
	/// Solves the determinant equation from:
	/// http://home.att.net/~srschmitt/circle3pts.html
	float x1 = p1.m_x;
	float y1 = p1.m_y;
	float x2 = p2.m_x;
	float y2 = p2.m_y;
	float x3 = p3.m_x;
	float y3 = p3.m_y;
	float x1y1_2 = (x1 * x1 + y1 * y1);
	float x2y2_2 = (x2 * x2 + y2 * y2);
	float x3y3_2 = (x3 * x3 + y3 * y3);

	float m11 = Determinant3x3(x1, y1, 1, x2, y2, 1, x3, y3, 1);
	float m12 = Determinant3x3(x1y1_2, y1, 1, x2y2_2, y2, 1, x3y3_2, y3, 1);
	float m13 = Determinant3x3(x1y1_2, x1, 1, x2y2_2, x2, 1, x3y3_2, x3, 1);
	float m14 = Determinant3x3(x1y1_2, x1, y1, x2y2_2, x2, y2, x3y3_2, x3, y3);

	center.m_x = 0.5f * (m12 / m11);
	center.m_y = -0.5f * (m13 / m11);

	return sqrtf((center.m_x * center.m_x) + (center.m_y * center.m_y) + (m14 / m11));
}

float Circle::Determinant3x3(float m00, float m01, float m02, 
							 float m10, float m11, float m12, 
							 float m20, float m21, float m22)
{
	return m00 * (m11 * m22 - m12 * m21) - m01 * (m10 * m22 - m12 * m20) + m02 * (m10 * m21 - m11 * m20);
}