/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file DelaunayTestRunner.cpp
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the DelaunayTestRunner class implementation.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#include "DelaunayTestRunner.h"

using namespace DelaunayTest;

DelaunayTestRunner::DelaunayTestRunner(void)
{
	m_entries.push_back(GlutWrapper::MenuEntry("Triangulate", DelaunayTest::Triangulate));
	m_entries.push_back(GlutWrapper::MenuEntry("Reset", DelaunayTest::Reset));
	m_entries.push_back(GlutWrapper::MenuEntry("Enable Circles", DelaunayTest::EnableTriangleCircles));
	m_entries.push_back(GlutWrapper::MenuEntry("Add Sites (+5)", DelaunayTest::AddSites));
	m_entries.push_back(GlutWrapper::MenuEntry("Remove Sites (-5)", DelaunayTest::RemoveSites));
}

DelaunayTestRunner::~DelaunayTestRunner(void)
{
}
	
void DelaunayTestRunner::InitializeOpengl(int width, int height)
{
	m_render.LoadPresetConfiguration(width, height);
}

void DelaunayTestRunner::Run(void)
{
	Render();
}

void DelaunayTestRunner::Reshape(int width, int height)
{
	m_render.LoadPresetConfiguration(width, height);
}

void DelaunayTestRunner::ProcessMenuEvents(int option)
{
	switch (option) 
	{
		case DelaunayTest::Triangulate : 
			m_logic.Triangulate(); 
			break;
	
		case DelaunayTest::Reset : 
			m_logic.Reset(); 
			break;

		case DelaunayTest::EnableTriangleCircles : 
			m_logic.EnableCircles(); 
			break;

		case DelaunayTest::AddSites	:	
			m_logic.AddSites(); 
			break;

		case DelaunayTest::RemoveSites	:	
			m_logic.RemoveSites(); 
			break;
	}
}

const std::vector<GlutWrapper::MenuEntry>& DelaunayTestRunner::GetMenuEntries(void) const
{
	return m_entries;
}

void DelaunayTestRunner::Render(void)
{
	m_render.BeginFrame();
	
	/// Render edges
	if (m_logic.IsTriangulated())
	{
		m_render.SetMaterial(1.0f, 0.0f, 0.0f);
		m_render.DrawLines(m_logic.GetLines());
	}

	/// Render circles
	if (m_logic.IsCirclesEnabled())
	{
		m_render.SetMaterial(0.0f, 0.0f, 1.0f);
		m_render.DrawCircles(m_logic.GetCircles());
	}

	/// Always render sites
	m_render.SetMaterial(0.0f, 1.0f, 0.0f);
	m_render.DrawPoints(m_logic.GetPoints());

	m_render.EndFrame();
}