/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file IRunner.h
/// This source file is part of 2dRender Delaunay Test.
/// The file holds the IRunner class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)

#ifndef _I_RUNNER_H_
#define _I_RUNNER_H_

namespace GlutWrapper
{

/// @brief Interface to a test runner. It's used by 
/// GlutWrapper::Window to run the test.
class IRunner
{
public:
	/// @brief Its used to configurate opengl.
	/// @param width render window width
	/// @param height render window height
	virtual void InitializeOpengl(int width, int height) = 0;

	/// @brief It executes the test.
	virtual void Run(void) = 0;

	/// @brief Callback notifies of render window reshape
	/// @param width new render window width
	/// @param height new render window height
	virtual void Reshape(int width, int height) = 0;

protected:	
	/// It can't be constructed
	IRunner(void){};

	/// It can`t be destroyed
	virtual ~IRunner(void) = 0 {};

private:
	/// @note hide copy constructor from interface user
	IRunner(const IRunner&);

	/// @note hide assignment operator from interface user
	IRunner& operator=(const IRunner&);

};

}
#endif